/*
 * CDDL HEADER START
 *
 * The contents of this file are subject to the terms of the
 * Common Development and Distribution License (the "License").
 * You may not use this file except in compliance with the License.
 *
 * You can obtain a copy of the license at usr/src/OPENSOLARIS.LICENSE
 * or http://www.opensolaris.org/os/licensing.
 * See the License for the specific language governing permissions
 * and limitations under the License.
 *
 * When distributing Covered Code, include this CDDL HEADER in each
 * file and include the License file at usr/src/OPENSOLARIS.LICENSE.
 * If applicable, add the following below this CDDL HEADER, with the
 * fields enclosed by brackets "[]" replaced with your own identifying
 * information: Portions Copyright [yyyy] [name of copyright owner]
 *
 * CDDL HEADER END
 */

/*
 * Copyright 2009 Sun Microsystems, Inc.  All rights reserved.
 * Use is subject to license terms.
 */

#pragma ident	"@(#)user_vtoc.h	1.5	09/05/26 SMI"

/*
 * User_vtoc:
 *
 *	The user vtoc library has to export 1 routine:
 *	user_read_vtoc which takes a file descriptor as an argument and
 *	returns a disko_vtoc structure allocated with alloc_disko_vtoc(N)
 *	where N is the number of partitions that the vtoc has.
 *
 *	This interface is evolving. Binary compatability between diskomizer
 *	builds should not be expected. Any code using this header file should
 *	be recompiled if a new version of diskomizer is installed.
 */

#ifndef _DISKOMIZER_USER_VTOC_H
#define	_DISKOMIZER_USER_VTOC_H

#ifdef	__cplusplus
extern "C" {
#endif

extern struct disko_vtoc *alloc_disko_vtoc(uint_t nparts);
typedef struct disko_vtoc *(*user_read_vtoc_t)(int fd);

#define	DISKO_VTOC_VERSION (0xD15C0000 + 3)

struct disko_partition {
	uint64_t p_tag;
	uint64_t p_flag;
	uint64_t p_start;
	uint64_t p_size;
};

struct disko_vtoc {
	int		disko_vtoc_version; /* set this to DISKO_VTOC_VERSION */
	int		disko_vtoc_size; /* sizeof (struct disko_vtoc) */
	int		disko_part_size; /* sizeof (struct disko_partition) */
	char		v_volume[LEN_DKL_VVOL + 1];
	char		v_asciilabel[LEN_DKL_ASCII + 1];
	uint_t		v_sectorsz;
	uint_t		v_nparts;
	uint_t		this_part; /* the partition number of this device */
	struct disko_partition  v_part[1];
};

#ifdef	__cplusplus
}
#endif

#endif /* _DISKOMIZER_USER_VTOC_H */
