/*
 * CDDL HEADER START
 *
 * The contents of this file are subject to the terms of the
 * Common Development and Distribution License (the "License").
 * You may not use this file except in compliance with the License.
 *
 * You can obtain a copy of the license at usr/src/OPENSOLARIS.LICENSE
 * or http://www.opensolaris.org/os/licensing.
 * See the License for the specific language governing permissions
 * and limitations under the License.
 *
 * When distributing Covered Code, include this CDDL HEADER in each
 * file and include the License file at usr/src/OPENSOLARIS.LICENSE.
 * If applicable, add the following below this CDDL HEADER, with the
 * fields enclosed by brackets "[]" replaced with your own identifying
 * information: Portions Copyright [yyyy] [name of copyright owner]
 *
 * CDDL HEADER END
 */

/*
 * Copyright 2009 Sun Microsystems, Inc.  All rights reserved.
 * Use is subject to license terms.
 */

#pragma ident	"@(#)user_vtoc.c	1.5	09/05/26 SMI"

/*
 * An example user_vtoc reader library for diskomizer. This is only intended to
 * be read as an example
 *
 * Complile with:
 *	SPARC v7
 * cc -G -xarch=v7 -I/opt/CTEdiskomizer/include \
 *	-L/opt/CTEdiskomizer/lib/sparc-xO4 \
 *	-xO4 -o /tmp/sparc/user_vtoc.so user_vtoc.c -lvtoc -lefi
 *
 *	SPARC v9
 * cc -G -xarch=v9 -I/opt/CTEdiskomizer/include \
 *	-L/opt/CTEdiskomizer/lib/sparcv9-xO4 \
 *	-xO4 -o /tmp/sparcv9/user_vtoc.so user_vtoc.c -lvtoc -lefi
 *
 *	intel pentium
 * cc -G -xarch=generic -I/opt/CTEdiskomizer/include \
 *	-L/opt/CTEdiskomizer/lib/pentium \
 *	-xO4 -o /tmp/pentium/user_vtoc.so user_vtoc.c -lvtoc -lefi
 *
 *	intel pentium_pro
 * cc -G -xarch=pentium_pro -I/opt/CTEdiskomizer/include \
 *	-L/opt/CTEdiskomizer/lib/pentium_pro \
 *	-xO4 -o /tmp/pentium_pro/user_vtoc.so user_vtoc.c -lvtoc -lefi
 *
 *	To use:
 *	set the option VTOC_USER_LIB to be the full path to the
 *	shared object.
 *		Eg VTOC_USER_LIB=/tmp/sparcv9/user_vtoc.so
 */

#include <string.h>
#include <sys/types.h>
#include <sys/dkio.h>
#include <sys/vtoc.h>
#include <sys/efi_partition.h>
#include <stdio.h>

#include <diskomizer/user_vtoc.h>

struct disko_vtoc *
user_read_vtoc(int fd)
{
	struct dk_gpt *efi = NULL;
	struct disko_vtoc *dv;
	int i;
	uint_t x;

	if ((x = efi_alloc_and_read(fd, &efi)) < 0) {
		perror("efi_alloc_and_read");
		fflush(stderr);
		return (NULL);
	}

	if (efi->efi_version != EFI_VERSION_CURRENT ||
	    efi->efi_part_size != sizeof (efi->efi_parts[1])) {
		fprintf(stderr, "version %#x read %#x\npsize %d read %d\n",
		    EFI_VERSION_CURRENT, efi->efi_version,
		    sizeof (efi->efi_parts[1]), efi->efi_part_size);
		fflush(stderr);
		/* return (NULL); */
	}

	if ((dv = alloc_disko_vtoc(efi->efi_nparts)) != NULL) {
		memset(&dv->v_volume[0], 0, sizeof (dv->v_volume));
		memset(&dv->v_asciilabel[0], 0, sizeof (dv->v_asciilabel));
		/*
		 * Diskomizer checks these values to verify there is no
		 * version mismatch. They *MUST* be filled in. If they do
		 * not match the ones diskomizer expects it will ignore the
		 * vtoc returned.
		 */
		dv->disko_vtoc_version = DISKO_VTOC_VERSION;
		dv->disko_vtoc_size = sizeof (*dv);
		dv->disko_part_size = sizeof (dv->v_part[0]);

		dv->v_sectorsz = efi->efi_lbasize;
		dv->v_nparts = efi->efi_nparts;
		dv->this_part = x;

		for (i = 0; i < efi->efi_nparts; i++) {
			dv->v_part[i].p_tag = efi->efi_parts[i].p_tag;
			dv->v_part[i].p_flag = efi->efi_parts[i].p_flag;
			dv->v_part[i].p_start = efi->efi_parts[i].p_start;
			dv->v_part[i].p_size = efi->efi_parts[i].p_size;
		}
	} else {
		fprintf(stderr, "unable to alloc %d parts\n", efi->efi_nparts);
		fflush(stderr);
	}
	efi_free(efi);
	return (dv);
}
